// Fragment shader for per-pixel Phong interpolation and shading.

// The "varying" keyword means that the parameter's value is interpolated
// between the nearby vertices.
varying vec3 N;
varying vec3 v;

in vec4 gl_FragCoord;
uniform sampler2D tex;

uniform float brightness;

void main()
{
    // The scene's ambient light.
    vec4 ambient = gl_LightModel.ambient * gl_FrontMaterial.ambient;

	// The normal vectors is generally not normalized after being
	// interpolated across a triangle.  Here we normalize it.
	vec3 Normal = normalize(N);

	// Since the vertex is in eye space, the direction to the
	// viewer is simply the normalized vector from v to the
	// origin.
	vec3 Viewer = -normalize(v);

	vec4 texel = texture2D(tex,gl_TexCoord[0].st);


	//////////// directional light

	// get direction and normalize
	vec3 DirectionalLight  = normalize(gl_LightSource[0].position.xyz);

	// compute halfway vector
	vec3 DirectionalHalf = normalize(Viewer+DirectionalLight);

	// fompute factor to prevent light leakage from below the surface
	float DirectionalB = 1.0;
	if(dot(Normal, DirectionalLight)<0.0) DirectionalB = 0.0;

	// compute diffuse and specular
	float directionalDiffuseShade = max(dot(Normal, DirectionalLight), 0.0);
	float directionalSpecularShade = pow(max(dot(DirectionalHalf, Normal), 0.0), gl_FrontMaterial.shininess);
	
	// Compute geometric terms of diffuse, specular, and ambient for Directional Light
	vec4 directionalDiffuse = directionalDiffuseShade * gl_FrontLightProduct[0].diffuse;
	vec4 directionalSpecular = directionalSpecularShade * gl_FrontLightProduct[0].specular;
	vec4 directionalAmbient = gl_FrontLightProduct[0].ambient;

	vec4 directionalComponent = directionalAmbient + DirectionalB * (directionalDiffuse + directionalSpecular);
	
	//////////// point light

	// Get the point light direction and normalize
	vec3 PointLight = normalize(gl_LightSource[1].position.xyz - v);

	// Compute halfway vector for point light
	vec3 PointHalf = normalize(Viewer+PointLight);

	// Compute factor to prevent light leakage from below the
	// surface for Point Light
	float PointB = 1.0;
	if(dot(Normal, PointLight)<0.0) PointB = 0.0;

	// Compute geometric terms of diffuse, specular, and ambient for Point Light
	vec4 pointDiffuse = max(dot(Normal, PointLight), 0.0) * gl_FrontLightProduct[1].diffuse;
	vec4 pointSpecular = pow(max(dot(PointHalf, Normal), 0.0), gl_FrontMaterial.shininess) * gl_FrontLightProduct[1].specular;
	vec4 pointAmbient = gl_FrontLightProduct[1].ambient;
	float pointDistance = length(gl_LightSource[1].position.xyz - v);
	vec4 pointComponent = (1.0 / (gl_LightSource[1].constantAttenuation + 
								  gl_LightSource[1].linearAttenuation * pointDistance + 
								  gl_LightSource[1].quadraticAttenuation * (pointDistance * pointDistance)))
						   * (pointAmbient + PointB * (pointDiffuse + pointSpecular));

	// apply blinn phong to texture
	gl_FragColor = texel * (gl_FrontMaterial.emission + ambient + directionalComponent + pointComponent);
	gl_FragColor = gl_FragColor * brightness;
}