#include "model.h"
#include <algorithm>
#include "modelerdraw.h"

using namespace std;

///////////// class Model ///////////////
Model::Model(const char* name) 
  : properties(name), 
	texture("brick.png"), 
	tessellationShader("tessellationShader.vert", "tessellationShader.frag") {}
GroupProperty* Model::getProperties() { return &properties; }
void Model::draw() {}
void Model::tick() {}
void Model::load() {}
ICamera* Model::getCamera() { return NULL; }

///////////// class Light ///////////////
Light::Light(const char* name, GLuint lightNumber,
						 float x, float y, float z,
						 float dr, float dg, float db,
						 float sr, float sg, float sb,
						 float ar, float ag, float ab) :
	Model(name),
    lightX("Light X", -10, 10, x, .1f),
	lightY("Light Y", -10, 10, y, .1f),
	lightZ("Light Z", -10, 10, z, .1f),
	diffuse("Diffuse", dr, dg, db),
	specular("Specular", sr, sg, sb),
	ambient("Ambient",  ar, ag, ab),
	isOn("This light is on", true)
	{


	properties.add(&isOn)
			  .add(&diffuse)
			  .add(&specular)
			  .add(&ambient)
			  .add(&lightX)
			  .add(&lightY)
			  .add(&lightZ);
	
	properties.setCollapsed(true);

	this->lightNumber = lightNumber;
}

void Light::draw(bool directional) {
	if (!isOn.getValue()) {
		// Set values to 0
		const float zeroes[4] = {0,0,0,0};
		glLightfv( lightNumber, GL_DIFFUSE, zeroes );
		glLightfv( lightNumber, GL_SPECULAR, zeroes );
		glLightfv( lightNumber, GL_AMBIENT, zeroes );
		return;
	}

	// Enable OpenGL lighting
	// TODO: check to see if it's already on
	glEnable( GL_LIGHTING );

	// Enable the light we're using
	// TODO: check to see if it's already on
	glEnable(lightNumber);

	// Create array containing light position
	GLfloat lightPositionArray[4] = {
		lightX.getValue(),
		lightY.getValue(),
		lightZ.getValue(),
		directional ? 0.f : 1.f
	};

	// Return if the light is directional but has no direction.
	if (directional && lightPositionArray[0] == 0
		&& lightPositionArray[1] == 0
		&& lightPositionArray[2] == 0) return;

	// Create array containing light diffuse color
	GLfloat lightDiffuseArray[4] = {
		diffuse.getRed(),
		diffuse.getGreen(),
		diffuse.getBlue(),
		1
	};

	// Create array for light's specular color
	GLfloat lightSpecularArray[4] = {
		specular.getRed(),
		specular.getGreen(),
		specular.getBlue(),
		1
	};

	// Create array for light's ambient color
	GLfloat lightAmbientArray[4] = {
		ambient.getRed(),
		ambient.getGreen(),
		ambient.getBlue(),
		1
	};

	// Send light to OpenGL
	glLightfv( lightNumber, GL_POSITION, lightPositionArray );
	glLightfv( lightNumber, GL_DIFFUSE, lightDiffuseArray );
	glLightfv( lightNumber, GL_SPECULAR, lightSpecularArray );
	glLightfv( lightNumber, GL_AMBIENT, lightAmbientArray );

	// Draw a marker for the light if allowed
	if (ModelerDrawState::Instance()->showMarkers) {
		const float lightRadius = .5f;
		const float directionalOrbit = 5.f;
		
		glPushMatrix();
			glTranslatef(lightPositionArray[0],
						 lightPositionArray[1],
						 lightPositionArray[2]);
			glDisable(GL_LIGHTING);
			glDisable(GL_TEXTURE_2D);
			glColor3f(1, 1, 1);
			glBegin(GL_LINES);
				glVertex3f(-lightRadius, 0, 0);
				glVertex3f(lightRadius, 0, 0);
				glVertex3f(0, -lightRadius, 0);
				glVertex3f(0, lightRadius, 0);
				glVertex3f(0, 0, -lightRadius);
				glVertex3f(0, 0, lightRadius);
			glEnd();
			glEnable(GL_LIGHTING);
		glPopMatrix();
	}
}

PointLight::PointLight(const char* name, GLuint lightNumber,
				float x, float y, float z,
				float dr, float dg, float db,
				float sr, float sg, float sb,
				float ar, float ag, float ab,
				float attA, float attB, float attC) :
	Light(name, lightNumber, x, y, z, dr, dg, db, sr, sg, sb, ar, ag, ab),
	attenA("Quadratic Attenuation ^ 5", 0, 1, attA, .05f),
	attenB("Linear Attenuation ^ 5", 0, 1, attB, .05f),
	attenC("Constant Attenuation ^ 5", 0, 1, attC, .05f)
{
	properties.add(&attenA).add(&attenB).add(&attenC);
}

void PointLight::draw() {
	// Set attenuation
	glLightf(lightNumber, GL_CONSTANT_ATTENUATION, pow(attenC.getValue(), 5));
	glLightf(lightNumber, GL_LINEAR_ATTENUATION, pow(attenB.getValue(), 5));
	glLightf(lightNumber, GL_QUADRATIC_ATTENUATION, pow(attenA.getValue(), 5));

	Light::draw(false);
}

/**
 * A directional light.
 */
DirectionalLight::DirectionalLight(const char* name, GLuint lightNumber,
				float x, float y, float z,
				float dr, float dg, float db,
				float sr, float sg, float sb,
				float ar, float ag, float ab) :
	Light(name, lightNumber, x, y, z, dr, dg, db, sr, sg, sb, ar, ag, ab)
{
}

void DirectionalLight::draw() {
	Light::draw(true);
}

Mouse::Mouse(const char* name) : Model(name),
Bend_Body_Front_Back("Bend Body Forward and Backward", -45.0f, 45.0f, 0.0f, 0.0f),
Bend_Body_Left_Right("Bend Body Left and Right", -45.0f, 45.0f, 0.0f, 0.0f),
move_Rshoulder_up_down("Move Left Shoulder Up and Down", 180.0f, 360.0f, 270.0f, 0.1f),
move_Rshoulder_forward_back("Move Left Shoulder Front and Back", 0.0f, 90.0f, 45.0f, 0.1f),
rotate_Rshoulder("Rotate Left Shoulder", 0.0f, 180.0f, 90.0f, 0.1f),
move_Relbow("Move Left Elbow", -90.0f, 95.0f, -90.0f, 0.1f),
move_Lshoulder_up_down("Move Right Shoulder Up and Down", 0.0f, 180.0f, 90.0f, 0.1f),
move_Lshoulder_forward_back("Move Right Shoulder Front and Back", 0.0f, 90.0f, 45.0f, 0.1f),
rotate_Lshoulder("Rotate Right Shoulder", 0.0f, -180.0f, -90.0f, 0.1f),
move_Lelbow("Move Right Elbow", 90.0f, -35.0f, 90.0f, 0.1f),
move_Rwaist_front_back("Move Left Leg Front to Back", 180.0f, 40.0f, 180.0f, 0.1f),
move_Rwaist_left_right("Move Left Leg Left to Right", 0.0f, -90.0f, 0.0f, 0.1f),
move_Lwaist_front_back("Move Right Leg Front to Back", 180.0f, 40.0f, 180.0f, 0.1f),
move_Lwaist_left_right("Move Right Leg Left to Right", 0.0f, 90.0f, 0.0f, 0.1f),
move_Rknee("Move Left Knee", 0.0f, 170.0f, 0.0f, 0.1f),
move_Lknee("move Right Knee", 180.0f, 10.0f, 180.0f, 0.1f),
Bong_sau("Bong Sau Angle?", 270.0f, 360.0f, 270.0f, 0.1f),
Bongsau("Perform Bong Sau?", false)

				

{
	properties.add(&Bend_Body_Front_Back);
	properties.add(&Bend_Body_Left_Right);
	properties.add(&move_Rshoulder_up_down);
	properties.add(&move_Rshoulder_forward_back);
	properties.add(&rotate_Rshoulder);
	properties.add(&move_Relbow);
	properties.add(&move_Lshoulder_up_down);
	properties.add(&move_Lshoulder_forward_back);
	properties.add(&rotate_Lshoulder);
	properties.add(&move_Lelbow);
	properties.add(&move_Rwaist_front_back);
	properties.add(&move_Rwaist_left_right);
	properties.add(&move_Lwaist_front_back);
	properties.add(&move_Lwaist_left_right);
	properties.add(&move_Rknee);
	properties.add(&move_Lknee);
	properties.add(&Bongsau);
	properties.add(&Bong_sau);
};



void Mouse::draw() {


	// body
	glPushMatrix();
	glRotatef(Bend_Body_Front_Back.getValue(), 1, 0, 0);
	glRotatef(Bend_Body_Left_Right.getValue(), 0, 0, 1);
	drawBox(0.2,2,0.2);
	

		// head
		glTranslated(0.1, 2, 0.1);
		glPushMatrix();
		drawSphere(0.5);
		glPopMatrix();

		// R shoulder
		glTranslated(-0.1, -2, -0.1);			// return to original
		glTranslated(0.1, 1.0, 0.1);
		glPushMatrix();
		drawSphere(0.2);

			// R upper arm
			glTranslated(0.1, 0.05, -0.05);
			glPushMatrix();
			if (Bongsau.getValue() == true) {
				glRotatef(Bong_sau.getValue(), 0, 0, 1);
			}
			else {
				glRotatef(move_Rshoulder_up_down.getValue(), 0, 0, 1);
			}
			glRotatef(move_Rshoulder_forward_back.getValue(), 1, 0, 0);
			glRotatef(rotate_Rshoulder.getValue(), 0, 1, 0);
			drawBox(0.1, 0.8, 0.1);
	

				// R elbow
				glRotatef(-270, 0, 0, 1);
				glTranslated(0.9, -0.05, 0.05);
				glPushMatrix();
				drawSphere(0.15);
				
					// R lower arm
					glTranslated(0.0, 0.05, -0.05);
					glPushMatrix();
					if (Bongsau.getValue() == true) {
						glRotatef(Bong_sau.getValue(), 0, 0, 1);
					}
					else {
						glRotatef(move_Relbow.getValue(), 0, 0, 1);
					}
					drawBox(0.1, 0.7, 0.1);
					glPopMatrix();

			
			glPopMatrix();

		glPopMatrix();

		// R waist
		glTranslated(-0.1, -2, -0.1);			// return to original
		glTranslated(0.1, 1.0, 0.1);
		glPushMatrix();
		drawSphere(0.2);

			// R upper leg
			glTranslated(0.05, 0.05, 0.0);
			glPushMatrix();
			glRotatef(move_Rwaist_front_back.getValue(), 1, 0, 0);
			glRotatef(move_Rwaist_left_right.getValue(), 0, 0, 1);
			drawBox(0.1, 0.8, 0.1);

				// R knee
				glRotatef(-270, 0, 0, 1);
				glTranslated(0.9, -0.05, 0.05);
				glPushMatrix();
				drawSphere(0.15);

					// R lower leg
					glTranslated(0.1, 0.05, 0.05);
					glPushMatrix();
					glRotatef(90, 1, 0, 0);
					glRotatef(-90, 0, 0, 1);
					glRotatef(move_Rknee.getValue(), 0, 0, 1);
					drawBox(0.1, 0.9, 0.1);
					glPopMatrix();

				glPopMatrix();

		glPopMatrix();

		// L shoulder
		glTranslated(0.0, 1.0, 0.0);
		glPushMatrix();
		drawSphere(0.2);

			// L upper arm
			glTranslated(-0.1, 0.00, -0.05);
			glPushMatrix();
			glRotatef(move_Lshoulder_up_down.getValue(), 0, 0, 1);
			glRotatef(move_Lshoulder_forward_back.getValue(), 1, 0, 0);
			glRotatef(rotate_Lshoulder.getValue(), 0, 1, 0);
			drawBox(0.1, 0.8, 0.1);
				
				// L elbow
				glRotatef(-90, 0, 0, 1);
				glTranslated(-0.9, 0.05, 0.05);
				glPushMatrix();
				drawSphere(0.15);

					// L lower arm
					glTranslated(-0.05, -0.05, -0.05);
					glPushMatrix();
					glRotatef(move_Lelbow.getValue(), 0, 0, 1);
					drawBox(0.1, 0.7, 0.1);
					glPopMatrix();

				glPopMatrix();

		glPopMatrix();


		// L waist
		glTranslated(-0.1, -2, -0.1);			// return to original
		glTranslated(0.1, 1.0, 0.1);
		glPushMatrix();
		drawSphere(0.2);

			// L upper leg
			glTranslated(-0.15, 0.0, 0.0);
			glPushMatrix();
			glRotatef(move_Lwaist_front_back.getValue(), 1, 0, 0);
			glRotatef(move_Lwaist_left_right.getValue(), 0, 0, 1);
			drawBox(0.1, 0.8, 0.1);

				// L knee
				glRotatef(-90, 0, 0, 1);
				glTranslated(-0.85, 0.05, 0.05);
				glPushMatrix();
				drawSphere(0.15);

				// L lower leg
				glTranslated(-0.1, 0.05, -0.05);
				glPushMatrix();
				glRotatef(90, 1, 0, 0);
				glRotatef(-90, 0, 0, 1);
				glRotatef(move_Lknee.getValue(), 0, 0, 1);
				drawBox(0.1, 0.9, 0.1);
				glPopMatrix();

			glPopMatrix();

		glPopMatrix();


	glPopMatrix();

	
};
